/*****************************************************************************
// Copyright (c) Lawrence Livermore National Security, LLC and other VisIt
// Project developers.  See the top-level LICENSE file for dates and other
// details.  No copyright assignment is required to contribute to VisIt.
*****************************************************************************/

#include <WDataPluginInfo.h>
#include <avtWDataFileFormat.h>
#include <avtMTSDFileFormatInterface.h>
#include <avtGenericDatabase.h>

// ****************************************************************************
//  Method:  WDataCommonPluginInfo::GetDatabaseType
//
//  Purpose:
//    Returns the type of a wdata database.
//
//  Programmer:  generated by xml2info
//  Creation:    omitted
//
// ****************************************************************************
DatabaseType
WDataCommonPluginInfo::GetDatabaseType()
{
    return DB_TYPE_MTSD;
}

// ****************************************************************************
//  Method: WDataCommonPluginInfo::SetupDatabase
//
//  Purpose:
//      Sets up a WData database.
//
//  Arguments:
//      list    A list of file names.
//      nList   The number of timesteps in list.
//      nBlocks The number of blocks in the list.
//
//  Returns:    A WData database from list.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************
avtDatabase *
WDataCommonPluginInfo::SetupDatabase(const char *const *list,
                                     int nList, int nBlock)
{
    int nTimestepGroups = nList / nBlock;
    avtMTSDFileFormat ***ffl = new avtMTSDFileFormat **[nTimestepGroups];
    for (int i = 0; i < nTimestepGroups; i++)
    {
        ffl[i] = new avtMTSDFileFormat *[nBlock];
        for (int j = 0; j < nBlock; j++)
        {
            ffl[i][j] = new avtWDataFileFormat(list[i * nBlock + j]);
        }
    }
    avtMTSDFileFormatInterface *inter = new avtMTSDFileFormatInterface(ffl, nTimestepGroups, nBlock);
    return new avtGenericDatabase(inter);
}
// ****************************************************************************
//  Method: WDataCommonPluginInfo::GetLicense
//
//  Purpose:
//      Gets the write options.
//
//  Programmer: generated by xml2info
//  Creation:   omitted
//
// ****************************************************************************

std::string
WDataCommonPluginInfo::GetLicense() const
{
    return std::string();
}
